;	NAME	assembly(16)
;	PUBLIC	fill_temp_buffer
;	PUBLIC	write_page
;	PUBLIC	write_lock_bits
;	PUBLIC  read_program_memory
;	RSEG	CODE


  #include "defines.h"
  #define _SPM    DW 0x95E8 ;Workaround for not having enhanced core Assembler in EWAAVR 2.25

.global write_page 
write_page:
        RCALL   WAIT_SPMEN  ;Wait for SPMEN flag cleared
        MOV     R31,R25         
        MOV     R30,R24     ;move adress to z pointer (R31=ZH R30=ZL)
        OUT 	_SFR_IO_ADDR(SPMCR), R22  ; argument 2 decides function (r22)
        SPM                 ;Store program memory
        .word   0xFFFF      ;For future part compatibility, instruction alignment
        NOP
        RJMP    WAIT_SPMEN ;Wait for SPMEN flag cleared

.global fill_temp_buffer
fill_temp_buffer:
        RCALL   WAIT_SPMEN  ;Wait for SPMEN flag cleared
        MOV     R31,R23     ;move adress to z pointer (R31=ZH R30=ZL)   
        MOV     R30,R22     
        MOV     R1,R25      ;move data to reg 0 and 1      
        MOV     R0,R24
        
        LDI     R18,(1<<SPMEN)
        OUT 	_SFR_IO_ADDR(SPMCR), R18   ; r18 decides function
        SPM                ;Store program memory
        .word     0xFFFF      ;For future part compatibility, instruction alignment
        NOP
        RJMP   WAIT_SPMEN  ;Wait for SPMEN flag cleared

.global read_program_memory
read_program_memory:
        RCALL    WAIT_SPMEN
        MOV     R31,R25     ;R31=ZH R30=ZL
        MOV     R30,R24     ;move adress to z pointer
        SBRC    R22,0       ;read lockbits? (second argument=0x09)
                            ;if so, place second argument in SPMEN register
        OUT 	_SFR_IO_ADDR(SPMCR), R22   ; r22 decides function
		
;If large memory (>64K) ELPM is needed to use RAMPZ                
#ifdef LARGE_MEMORY 
        ELPM                ;read LSB        
#else
        LPM                 
#endif

        MOV     R24,R0      ;read LSB           
        INC     R30
		
;If large memory (>64K) ELPM is needed to use RAMPZ        
#ifdef LARGE_MEMORY 
        ELPM                ;read LSB        
#else
        LPM                 
#endif

        MOV     R25,R0      ;read MSB (ignored when reading lockbits)
        RET

.global write_lock_bits
write_lock_bits:
        RCALL   WAIT_SPMEN  ;Wait for SPMEN flag cleared
        MOV     R0,R24   
        LDI     R18,((1<<BLBSET)|(1<<SPMEN))

        OUT 	_SFR_IO_ADDR(SPMCR), R18   ; r18 decides function
        SPM                ;write lockbits
        .word      0xFFFF      ;For future part compatibility, instruction alignment
        NOP
        
        RJMP   WAIT_SPMEN  ;Wait for SPMEN flag cleared
        

WAIT_SPMEN:
		MOV		R0, R18
		IN	 	R18, _SFR_IO_ADDR(SPMCR)   ; r18 decides function
        SBRC    R18,SPMEN
        RJMP    WAIT_SPMEN  ;Wait for SPMEN flag cleared
        MOV		R18, R0

        RET



